// -*- mode: c++ basic-offset: 4; indent-tabs-mode: nil; -*-
#ifndef DISKCACHE_API__SDGTX_FILENAME_HANDLER_HH
#define DISKCACHE_API__SDGTX_FILENAME_HANDLER_HH

#include "config.h"

#include "general/types.hh"

#include "FilenameHandler.hh"

namespace diskCache
{
  //---------------------------------------------------------------------
  /// \brief Interface for handling filenames of the form S-D-G-T.x
  ///
  /// Files that are handled by this filename handler conform to
  /// to the ligo Technical Note T010150-00 with the one modification
  /// that they can have an extension other than '.gwf'.
  //---------------------------------------------------------------------
  class SDGTxFilenameHandler
    : public FilenameHandler
  {
  public:
    //-------------------------------------------------------------------
    /// \brief Virtual constructor of Hash entry
    //-------------------------------------------------------------------
    virtual Hash* CreateHash( dir_info_type DirInfo ) const;

#if ENABLE_MOUNT_POINT_CACHE
    //-------------------------------------------------------------------
    /// \brief Virtual constructor of MountPointCache::Info entry
    //-------------------------------------------------------------------
    virtual MountPointCache* CreateMountPointCache( ) const;
#endif /* old */

    //-------------------------------------------------------------------
    // Take definition from base class
    //-------------------------------------------------------------------
    virtual bool
    IsFile( const std::string& Filename,
            FilenameHandler::IsFileData& FileData ) const;

    //-------------------------------------------------------------------
    /// \brief Register filename handler
    //-------------------------------------------------------------------
    static bool Register( );

    //-------------------------------------------------------------------
    /// \brief Unregister filename handler
    //-------------------------------------------------------------------
    static bool Unregister( );

    //-------------------------------------------------------------------
    /// \brief Retrieve the key that identifies this object.
    //-------------------------------------------------------------------
    static FilenameHandler* Key( );

  protected:
    class FileData
    {
    public:
      typedef INT_4U g_type;
      typedef INT_4U t_type;

      const std::string	m_S;	///< Site information 
      const std::string m_D;	///< Description
      const g_type	m_G;	///< GPS start time
      const t_type	m_T;	///< Duration
      const std::string m_x;	///< File extension

      //-----------------------------------------------------------------
      /// \brief Constructor
      ///
      /// \param[in] S
      ///     The source of the data which is to include uppercase
      ///     letters in alphabetic order indicating the detecor
      ///     site or sites.
      ///     e.g: 'G', 'HL'
      /// \param[in] D
      ///     The description of the contents of a file.
      /// \param[in] G
      ///     The GPS start time of the data set in integral seconds.
      /// \param[in] T
      ///     The total time interval covered by the file
      ///     in integral seconds.
      /// \param[in] X
      ///     The file extension associated with the file.
      ///     e.g. '.gwf'
      //-----------------------------------------------------------------
      FileData( const std::string& S,
		const std::string& D,
	        g_type G,
		t_type T,
		const std::string& X );
    };
  private:
    //-------------------------------------------------------------------
    /// \brief Update the file pattern.
    //-------------------------------------------------------------------
    void update_regular_expression( ) const;
  };

  //---------------------------------------------------------------------
  /// Creation of a FileData object.
  //---------------------------------------------------------------------
  inline SDGTxFilenameHandler::FileData::
  FileData( const std::string& S,
	    const std::string& D,
	    g_type G,
	    t_type T,
	    const std::string& X )
    : m_S( S ),
      m_D( D ),
      m_G( G ),
      m_T( T ),
      m_x( X )
  {
  }
  
}
#endif /* DISKCACHE_API__SDGTX_FILENAME_HANDLER_HH */
