#include <algorithm>
#include <sstream>

#include "general/AtExit.hh"
#include "general/ErrorLog.hh"

#include "genericAPI/StatVFSDirect.hh"
#include "genericAPI/StatVFSPool.hh"

using General::StdErrLog;

static bool initialize( );

namespace GenericAPI
{
  // Instantiate generic to Singleton class methods:   
  // see "general" library   
  SINGLETON_TS_INST( StatVFSPool );

  StatVFSPool::pool_type StatVFSPool::m_in_use;
  StatVFSPool::pool_type StatVFSPool::m_available;
  MutexLock::lock_type StatVFSPool::m_pool_baton = PTHREAD_MUTEX_INITIALIZER;
  StatVFSPool::user_init_cb StatVFSPool::m_user_init_cb = NULL;
  bool StatVFSPool::m_initialized = initialize( );

  StatVFSPool::
  StatVFSPool( )
  {
    if ( StdErrLog.IsOpen( ) )
    {
      std::ostringstream	msg;
      
      msg << "StatVFSPool::StatVFSPool";
      StdErrLog( General::ErrorLog::DEBUG,
		 __FILE__, __LINE__,
		 msg.str( ) );
    }
  }

  void StatVFSPool::
  Cleanup( )
  {
    static bool has_been_cleaned = false;
    if ( has_been_cleaned == false )
    {
      MutexLock	lock( m_pool_baton );

      if ( has_been_cleaned == false )
      {
	if ( StdErrLog.IsOpen( ) )
	{
	  std::ostringstream	msg;
      
	  msg << "StatVFSPool::~StatVFSPool";
	  StdErrLog( General::ErrorLog::DEBUG,
		     __FILE__, __LINE__,
		     msg.str( ) );
	}
	for (  pool_type::const_iterator
		 cur = m_available.begin( ),
		 last = m_available.end( );
	       cur != last;
	       ++cur )
	{
	  delete *cur;
	}
	m_available.erase( m_available.begin( ), m_available.end( ) );
#if 0
	for (  pool_type::const_iterator
		 cur = m_in_use.begin( ),
		 last = m_in_use.end( );
	       cur != last;
	       ++cur )
	{
	  delete *cur;
	}
#endif /* 0 */
	m_in_use.erase( m_in_use.begin( ), m_in_use.end( ) );
	has_been_cleaned = true;
      } // if ( has_been_cleaned == false ) (after Mutex lock)
    } // if ( has_been_cleaned == false )
  }

  const StatVFSPool::info_type StatVFSPool::
  Base( )
  {
    static info_base_type	stat_vfs_base;

    return &stat_vfs_base;
  }

  void StatVFSPool::
  Destroy( StatVFSPool::info_type Key )
  {
    MutexLock	lock( m_pool_baton );

    if ( StdErrLog.IsOpen( ) )
    {
      std::ostringstream	msg;
      
      msg << "StatVFSPool::destroy: Key: " << (void*)Key;
      if ( Key )
      {
	msg << Key->Debug( );
      }
      StdErrLog( General::ErrorLog::DEBUG,
		 __FILE__, __LINE__,
		 msg.str( ) );
    }
    //-------------------------------------------------------------------
    // Take out of m_in_use list
    //-------------------------------------------------------------------
    pool_type::size_type	size = m_in_use.size( );

    /* erase the element */
    m_in_use.remove( Key );
    if ( size != m_in_use.size( ) )
    {
      //-----------------------------------------------------------------
      // Destroy the process
      //-----------------------------------------------------------------
      if ( StdErrLog.IsOpen( ) )
      {
	std::ostringstream	msg;
      
	msg << "StatVFSPool::release: deleting: Key: " << (void*)Key;
	if ( Key )
	{
	  msg << Key->Debug( );
	}
	StdErrLog( General::ErrorLog::DEBUG,
		   __FILE__, __LINE__,
		   msg.str( ) );
      }
      delete Key;
    }
  }

  void StatVFSPool::
  Release( StatVFSPool::info_type Key )
  {
    MutexLock	lock( m_pool_baton );

    if ( StdErrLog.IsOpen( ) )
    {
      std::ostringstream	msg;
      
      msg << "StatVFSPool::release: " << Key->Debug( );
      StdErrLog( General::ErrorLog::DEBUG,
		 __FILE__, __LINE__,
		 msg.str( ) );
    }
    //-------------------------------------------------------------------
    // Take out of m_in_use list
    //-------------------------------------------------------------------
    pool_type::size_type size = m_in_use.size( );

    /* erase the element */
    m_in_use.remove( Key );
    if ( size != m_in_use.size( ) )
    {
      //-----------------------------------------------------------------
      // Put into m_available queue
      //-----------------------------------------------------------------
      if ( StdErrLog.IsOpen( ) )
      {
	std::ostringstream	msg;
      
	msg << "StatVFSPool::release: returned to a available pool: "
	    << Key->Debug( );
	StdErrLog( General::ErrorLog::DEBUG,
		   __FILE__, __LINE__,
		   msg.str( ) );
      }
      if ( General::AtExit::IsExiting( ) )
      {
	delete Key;
      }
      else
      {
	m_available.push_back( Key );
      }
    }
  }

  StatVFSPool::info_type StatVFSPool::
  Request( )
  {
    MutexLock	lock( m_pool_baton );
    info_type retval = NULL;

    if ( m_available.size( ) == 0 )
    {
      //-----------------------------------------------------------------
      // Create new thread
      //-----------------------------------------------------------------
      retval = Base( )->vnew( );
      if ( m_user_init_cb )
      {
	(*m_user_init_cb)( retval );
      }
      retval->Init( );
      if ( StdErrLog.IsOpen( ) )
      {
	std::ostringstream	msg;
	
	msg << "StatVFSPool::request: Created new " << retval->Debug( );
	StdErrLog( General::ErrorLog::DEBUG,
		   __FILE__, __LINE__,
		   msg.str( ) );
      }
    }
    else
    {
      //-----------------------------------------------------------------
      // Take one from the m_available list
      //-----------------------------------------------------------------
      retval = m_available.front( );
      m_available.pop_front( );
      if ( StdErrLog.IsOpen( ) )
      {
	std::ostringstream	msg;
	
	msg << "StatVFSPool::request: Reusing " << retval->Debug( );
	StdErrLog( General::ErrorLog::DEBUG,
		   __FILE__, __LINE__,
		   msg.str( ) );
      }
    }
    m_in_use.push_back( retval );
    return retval;
  }
}

static bool
initialize( )
{
  General::AtExit::Append( GenericAPI::StatVFSPool::Cleanup,
			   "GenericAPI::StatVFSPool::Cleanup",
			   120 );
  return true;
}
